@@ -28,6 +28,19 @@ using std::for_each;
 static TSK_HDB_INFO * m_NSRLDb = NULL;
 static std::vector<TSK_HDB_INFO *> m_knownbads;
 
+/*
+* JNI file handle structure encapsulates both
+* TSK_FS_FILE file handle and TSK_FS_ATTR attribute
+* to support multiple attributes for the same file.
+* TSK_FS_FILE still needs be maintained for opening and closing.
+*/
+typedef struct {
+    uint32_t tag; 
+    TSK_FS_FILE *fs_file; 
+    TSK_FS_ATTR *fs_attr; 
+} TSK_JNI_FILEHANDLE;
+#define TSK_JNI_FILEHANDLE_TAG 0x10101214
+
 /**
 * Sets flag to throw an TskCoreException back up to the Java code with a specific message.
 * Note: exception is thrown to Java code after the native function returns
@@ -93,6 +106,7 @@ castImgInfo(JNIEnv * env, jlong ptr)
     TSK_IMG_INFO *lcl = (TSK_IMG_INFO *) ptr;
     if (lcl->tag != TSK_IMG_INFO_TAG) {
         setThrowTskCoreError(env, "Invalid IMG_INFO object");
+        return 0;
     }
     return lcl;
 }
@@ -104,6 +118,7 @@ castVsInfo(JNIEnv * env, jlong ptr)
     TSK_VS_INFO *lcl = (TSK_VS_INFO *) ptr;
     if (lcl->tag != TSK_VS_INFO_TAG) {
         setThrowTskCoreError(env, "Invalid VS_INFO object");
+        return 0;
     }
 
     return lcl;
@@ -115,6 +130,7 @@ castVsPartInfo(JNIEnv * env, jlong ptr)
     TSK_VS_PART_INFO *lcl = (TSK_VS_PART_INFO *) ptr;
     if (lcl->tag != TSK_VS_PART_INFO_TAG) {
         setThrowTskCoreError(env, "Invalid VS_PART_INFO object");
+        return 0;
     }
 
     return lcl;
@@ -126,17 +142,19 @@ castFsInfo(JNIEnv * env, jlong ptr)
     TSK_FS_INFO *lcl = (TSK_FS_INFO *) ptr;
     if (lcl->tag != TSK_FS_INFO_TAG) {
         setThrowTskCoreError(env, "Invalid FS_INFO object");
+        return 0;
     }
     return lcl;
 }
 
 
-static TSK_FS_FILE *
+static TSK_JNI_FILEHANDLE *
 castFsFile(JNIEnv * env, jlong ptr)
 {
-    TSK_FS_FILE *lcl = (TSK_FS_FILE *) ptr;
-    if (lcl->tag != TSK_FS_FILE_TAG) {
-        setThrowTskCoreError(env, "Invalid FS_FILE object");
+    TSK_JNI_FILEHANDLE *lcl = (TSK_JNI_FILEHANDLE *) ptr;
+    if (lcl->tag != TSK_JNI_FILEHANDLE_TAG) {
+        setThrowTskCoreError(env, "Invalid TSK_JNI_FILEHANDLE object");
+        return 0;
     }
     return lcl;
 }
@@ -148,6 +166,7 @@ castCaseDb(JNIEnv * env, jlong ptr)
     if (lcl->m_tag != TSK_CASE_DB_TAG) {
         setThrowTskCoreError(env,
             "Invalid TskCaseDb object");
+        return 0;
     }
 
     return lcl;
@@ -223,6 +242,10 @@ JNIEXPORT void JNICALL
     jclass obj, jlong caseHandle) {
 
     TskCaseDb *tskCase = castCaseDb(env, caseHandle);
+    if (tskCase == 0) {
+        //exception already set
+        return;
+    }
 
     delete tskCase;
     return;
@@ -419,6 +442,10 @@ JNIEXPORT jlong JNICALL
     jboolean isCopy;
 
     TskCaseDb *tskCase = castCaseDb(env, caseHandle);
+    if (tskCase == 0) {
+        //exception already set
+        return 0;
+    }
 
     char envstr[32];
     snprintf(envstr, 32, "TZ=%s", env->GetStringUTFChars(timezone,
@@ -477,8 +504,6 @@ JNIEXPORT void JNICALL
     //change to false if hashes aren't needed
     tskAuto->hashFiles(false);
 
-    
-
     // move the strings into the C++ world
 
     // get pointers to each of the file names
@@ -676,6 +701,10 @@ JNIEXPORT jlong JNICALL
 JNIEXPORT jlong JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_openVsNat
     (JNIEnv * env, jclass obj, jlong a_img_info, jlong vsOffset) {
     TSK_IMG_INFO *img_info = castImgInfo(env, a_img_info);
+    if (img_info == 0) {
+        //exception already set
+        return 0;
+    }
     TSK_VS_INFO *vs_info;
 
     vs_info = tsk_vs_open(img_info, vsOffset, TSK_VS_TYPE_DETECT);
@@ -699,6 +728,10 @@ Java_org_sleuthkit_datamodel_SleuthkitJNI_openVolNat(JNIEnv * env,
     jclass obj, jlong a_vs_info, jlong vol_id)
 {
     TSK_VS_INFO *vs_info = castVsInfo(env, a_vs_info);
+    if (vs_info == 0) {
+        //exception already set
+        return 0;
+    }
     const TSK_VS_PART_INFO *vol_part_info;
 
     vol_part_info = tsk_vs_part_get(vs_info, (TSK_PNUM_T) vol_id);
@@ -720,6 +753,10 @@ Java_org_sleuthkit_datamodel_SleuthkitJNI_openVolNat(JNIEnv * env,
 JNIEXPORT jlong JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_openFsNat
     (JNIEnv * env, jclass obj, jlong a_img_info, jlong fs_offset) {
     TSK_IMG_INFO *img_info = castImgInfo(env, a_img_info);
+    if (img_info == 0) {
+        //exception already set
+        return 0;
+    }
     TSK_FS_INFO *fs_info;
 
     fs_info =
@@ -734,25 +771,54 @@ JNIEXPORT jlong JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_openFsNat
 
 /*
  * Open the file with the given id in the given file system
- * @return the created TSK_FS_FILE pointer
+ * @return the created TSK_JNI_FILEHANDLE pointer, set throw exception on error
  * @param env pointer to java environment this was called from
  * @param obj the java object this was called from
  * @param a_fs_info the pointer to the parent file system object
- * @param file_id id of the file to open 
+ * @param file_id id of the file to open
+ * @param attr_type type of the file attribute to open
+ * @param attr_id id of the file attribute to open
  */
 JNIEXPORT jlong JNICALL
 Java_org_sleuthkit_datamodel_SleuthkitJNI_openFileNat(JNIEnv * env,
-    jclass obj, jlong a_fs_info, jlong file_id)
+    jclass obj, jlong a_fs_info, jlong file_id, jint attr_type, jint attr_id)
 {
     TSK_FS_INFO *fs_info = castFsInfo(env, a_fs_info);
-    TSK_FS_FILE *file_info;
+    if (fs_info == 0) {
+        //exception already set
+        return 0;
+    }
 
+    //allocate file handle structure
+    TSK_JNI_FILEHANDLE * fileHandle = 
+        (TSK_JNI_FILEHANDLE *) tsk_malloc(sizeof(TSK_JNI_FILEHANDLE));
+    if (fileHandle == NULL) {
+        setThrowTskCoreError(env, "Could not allocate memory for TSK_JNI_FILEHANDLE");
+        return 0;
+    }
+	
+    TSK_FS_FILE *file_info;
+    //open file
     file_info = tsk_fs_file_open_meta(fs_info, NULL, (TSK_INUM_T) file_id);
     if (file_info == NULL) {
         setThrowTskCoreError(env, tsk_error_get());
+        return 0;
     }
 
-    return (jlong) file_info;
+    //open attribute
+    const TSK_FS_ATTR * tsk_fs_attr = 
+        tsk_fs_file_attr_get_type(file_info, (TSK_FS_ATTR_TYPE_ENUM)attr_type, (uint16_t)attr_id, 1);
+    if (tsk_fs_attr == NULL) {
+        tsk_fs_file_close(file_info);
+        setThrowTskCoreError(env, tsk_error_get());
+        return 0;
+    }
+
+    fileHandle->tag = TSK_JNI_FILEHANDLE_TAG;
+    fileHandle->fs_file = file_info;
+    fileHandle->fs_attr = const_cast<TSK_FS_ATTR*>(tsk_fs_attr);
+
+    return (jlong)fileHandle;
 }
 
 
@@ -811,6 +877,11 @@ Java_org_sleuthkit_datamodel_SleuthkitJNI_readImgNat(JNIEnv * env,
     }
 
     TSK_IMG_INFO *img_info = castImgInfo(env, a_img_info);
+    if (img_info == 0) {
+        free(buf);
+        //exception already set
+        return -1;
+    }
 
     ssize_t bytesread =
         tsk_img_read(img_info, (TSK_OFF_T) offset, buf, (size_t) len);
@@ -855,6 +926,11 @@ Java_org_sleuthkit_datamodel_SleuthkitJNI_readVsNat(JNIEnv * env,
         return -1;
     }
     TSK_VS_INFO *vs_info = castVsInfo(env, a_vs_info);
+    if (vs_info == 0) {
+        //exception already set
+        free(buf);
+        return -1;
+    }
 
     ssize_t bytesread = tsk_vs_read_block(vs_info, (TSK_DADDR_T) offset, buf,
         (size_t) len);
@@ -899,7 +975,11 @@ Java_org_sleuthkit_datamodel_SleuthkitJNI_readVolNat(JNIEnv * env,
     }
 
     TSK_VS_PART_INFO *vol_part_info = castVsPartInfo(env, a_vol_info);
-
+    if (vol_part_info == 0) {
+        free(buf);
+        //exception already set
+        return -1;
+    }
     ssize_t bytesread =
         tsk_vs_part_read(vol_part_info, (TSK_OFF_T) offset, buf,
         (size_t) len);
@@ -944,6 +1024,11 @@ Java_org_sleuthkit_datamodel_SleuthkitJNI_readFsNat(JNIEnv * env,
         return -1;
     }
     TSK_FS_INFO *fs_info = castFsInfo(env, a_fs_info);
+    if (fs_info == 0) {
+        free(buf);
+        //exception already set
+        return -1;
+    }
 
     ssize_t bytesread =
         tsk_fs_read(fs_info, (TSK_OFF_T) offset, buf, (size_t) len);
@@ -975,14 +1060,14 @@ Java_org_sleuthkit_datamodel_SleuthkitJNI_readFsNat(JNIEnv * env,
  * @return number of bytes read, or -1 on error
  * @param env pointer to java environment this was called from
  * @param obj the java object this was called from
- * @param a_file_info the pointer to the file object
+ * @param a_file_handle the pointer to the TSK_JNI_FILEHANDLE object
  * @param jbuf jvm allocated buffer to read to
  * @param offset the offset in bytes to start at
  * @param len number of bytes to read
  */
 JNIEXPORT jint JNICALL
 Java_org_sleuthkit_datamodel_SleuthkitJNI_readFileNat(JNIEnv * env,
-    jclass obj, jlong a_file_info, jbyteArray jbuf, jlong offset, jlong len)
+    jclass obj, jlong a_file_handle, jbyteArray jbuf, jlong offset, jlong len)
 {
     char *buf = (char *) tsk_malloc((size_t) len);
     if (buf == NULL) {
@@ -990,19 +1075,26 @@ Java_org_sleuthkit_datamodel_SleuthkitJNI_readFileNat(JNIEnv * env,
         return -1;
     }
 
-    TSK_FS_FILE *file_info = castFsFile(env, a_file_info);
+    const TSK_JNI_FILEHANDLE *file_handle = castFsFile(env, a_file_handle);
+    if (file_handle == 0) {
+        free(buf);
+        //exception already set
+        return -1;
+    }
 
-    ssize_t bytesread =
-        tsk_fs_file_read(file_info, (TSK_OFF_T) offset, buf, (size_t) len,
+    TSK_FS_ATTR * tsk_fs_attr = file_handle->fs_attr;
+
+    //read attribute
+    ssize_t bytesread = tsk_fs_attr_read(tsk_fs_attr,  (TSK_OFF_T) offset, buf, (size_t) len,
         TSK_FS_FILE_READ_FLAG_NONE);
     if (bytesread == -1) {
-		free(buf);
+        free(buf);
         setThrowTskCoreError(env, tsk_error_get());
-		return -1;
+        return -1;
     }
 
     // package it up for return
-	// adjust number bytes to copy
+    // adjust number bytes to copy
 	ssize_t copybytes = bytesread;
 	jsize jbuflen = env->GetArrayLength(jbuf);
 	if (jbuflen < copybytes)
@@ -1028,6 +1120,10 @@ Java_org_sleuthkit_datamodel_SleuthkitJNI_closeImgNat(JNIEnv * env,
     jclass obj, jlong a_img_info)
 {
     TSK_IMG_INFO *img_info = castImgInfo(env, a_img_info);
+    if (img_info == 0) {
+        //exception already set
+        return;
+    }
     tsk_img_close(img_info);
 }
 
@@ -1040,6 +1136,10 @@ Java_org_sleuthkit_datamodel_SleuthkitJNI_closeImgNat(JNIEnv * env,
 JNIEXPORT void JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_closeVsNat
     (JNIEnv * env, jclass obj, jlong a_vs_info) {
     TSK_VS_INFO *vs_info = castVsInfo(env, a_vs_info);
+    if (vs_info == 0) {
+        //exception already set
+        return;
+    }
     tsk_vs_close(vs_info);
 }
 
@@ -1052,6 +1152,10 @@ JNIEXPORT void JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_closeVsNat
 JNIEXPORT void JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_closeFsNat
     (JNIEnv * env, jclass obj, jlong a_fs_info) {
     TSK_FS_INFO *fs_info = castFsInfo(env, a_fs_info);
+    if (fs_info == 0) {
+        //exception already set
+        return;
+    }
     tsk_fs_close(fs_info);
 }
 
@@ -1065,8 +1169,19 @@ JNIEXPORT void JNICALL
 Java_org_sleuthkit_datamodel_SleuthkitJNI_closeFileNat(JNIEnv * env,
     jclass obj, jlong a_file_info)
 {
-    TSK_FS_FILE *file_info = castFsFile(env, a_file_info);
-    tsk_fs_file_close(file_info);
+    TSK_JNI_FILEHANDLE *file_handle = castFsFile(env, a_file_info);
+    if (file_handle == 0) {
+        //exception already set
+        return;
+    }
+	
+    TSK_FS_FILE * file_info = file_handle->fs_file;
+    tsk_fs_file_close(file_info); //also closes the attribute
+
+    file_handle->fs_file = NULL;
+    file_handle->fs_attr = NULL;
+    file_handle->tag = 0;
+    free (file_handle);
 }
 
 /*
@@ -1196,3 +1311,37 @@ JNIEXPORT jint JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_getIndexSizeNat
     tsk_hdb_close(temp);
     return -1;
 }
+
+
+/*
+ * Query and get size of the device (such as physical disk, or image) pointed by the path
+ * Might require elevated priviletes to work (otherwise will error)
+ * @param env pointer to java environment this was called from
+ * @param obj the java object this was called from
+ * @param devPathJ the device path
+ * @return size of device, set throw jni exception on error
+ */
+JNIEXPORT jlong JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_findDeviceSizeNat
+  (JNIEnv * env, jclass obj, jstring devPathJ) {
+     
+      jlong devSize = 0;
+      const char* devPath = env->GetStringUTFChars(devPathJ, 0);
+
+      // open the image to get the size
+      TSK_IMG_INFO * img_info =
+        tsk_img_open_utf8_sing(devPath, TSK_IMG_TYPE_DETECT, 0);
+      if (img_info == NULL) {
+        setThrowTskCoreError(env, tsk_error_get());
+        env->ReleaseStringUTFChars(devPathJ , devPath); 
+        return -1;
+      }
+
+      TSK_OFF_T imgSize = img_info->size;
+      devSize = imgSize;
+
+      //cleanup
+      tsk_img_close(img_info);
+      env->ReleaseStringUTFChars(devPathJ , devPath); 
+
+      return devSize;
+}
