@@ -31,9 +31,11 @@ import org.sleuthkit.datamodel.TskData.FileKnown;
 public abstract class FsContent extends AbstractFile {
 
 	///read only database tsk_files fields
-	protected final long fs_obj_id, meta_addr, attr_type, attr_id, meta_type, dir_type, dir_flags,
-			meta_flags, size, ctime, crtime, atime, mtime, uid, gid, mode;
-
+	protected final long fs_obj_id, meta_addr, size, ctime, crtime, atime, mtime;
+	protected final int uid, gid;
+	protected final short attr_type, attr_id, meta_type, dir_type, mode;
+	protected final short dir_flags, meta_flags;
+	
 	/*
 	 * path of parent directory
 	 */
@@ -42,16 +44,13 @@ public abstract class FsContent extends AbstractFile {
 	/**
 	 * known status in database
 	 */
-	protected long known;
+	protected byte known;
 	/*
 	 * md5 hash
 	 */
 	protected String md5Hash;
 	///other members
-	/*
-	 * Unique path containing image and volume
-	 */
-	protected String unique_path;
+
 	/**
 	 * parent file system
 	 */
@@ -88,8 +87,8 @@ public abstract class FsContent extends AbstractFile {
 	 * @param md5Hash
 	 */
 	FsContent(SleuthkitCase db, long obj_id, String name, long fs_obj_id, long meta_addr,
-			long attr_type, long attr_id, long meta_type, long dir_type, long dir_flags,
-			long meta_flags, long size, long ctime, long crtime, long atime, long mtime, long uid, long gid, long mode, long known,
+			short attr_type, short attr_id, short meta_type, short dir_type, short dir_flags,
+			short meta_flags, long size, long ctime, long crtime, long atime, long mtime, int uid, int gid, short mode, byte known,
 			String parent_path, String md5Hash) {
 		super(db, obj_id, name, TskData.TSK_DB_FILES_TYPE_ENUM.FS);
 		this.fs_obj_id = fs_obj_id;
@@ -140,7 +139,7 @@ public abstract class FsContent extends AbstractFile {
 	 * 
 	 * @param known 
 	 */
-	void setKnown(long known) {
+	void setKnown(byte known) {
 		this.known = known;
 	}
 
@@ -148,7 +147,8 @@ public abstract class FsContent extends AbstractFile {
 	public int read(byte[] buf, long offset, long len) throws TskCoreException {
 		synchronized (this) {
 			if (fileHandle == 0) {
-				fileHandle = SleuthkitJNI.openFile(parentFileSystem.getFileSystemHandle(), meta_addr);
+				fileHandle = 
+						SleuthkitJNI.openFile(parentFileSystem.getFileSystemHandle(), meta_addr, attr_type, attr_id);
 			}
 		}
 		return SleuthkitJNI.readFile(fileHandle, buf, offset, len);
@@ -192,7 +192,7 @@ public abstract class FsContent extends AbstractFile {
 	 *
 	 * @return attribute type
 	 */
-	public long getAttr_type() {
+	public short getAttr_type() {
 		return attr_type;
 	}
 
@@ -201,7 +201,7 @@ public abstract class FsContent extends AbstractFile {
 	 *
 	 * @return attribute id
 	 */
-	public long getAttr_id() {
+	public short getAttr_id() {
 		return attr_id;
 	}
 
@@ -210,7 +210,7 @@ public abstract class FsContent extends AbstractFile {
 	 *
 	 * @return meta data type
 	 */
-	public long getMeta_type() {
+	public short getMeta_type() {
 		return meta_type;
 	}
 
@@ -228,7 +228,7 @@ public abstract class FsContent extends AbstractFile {
 	 *
 	 * @return directory type id
 	 */
-	public long getDir_type() {
+	public short getDir_type() {
 		return dir_type;
 	}
 
@@ -246,7 +246,7 @@ public abstract class FsContent extends AbstractFile {
 	 *
 	 * @return directory flags
 	 */
-	public long getDir_flags() {
+	public short getDir_flags() {
 		return dir_flags;
 	}
 
@@ -273,7 +273,7 @@ public abstract class FsContent extends AbstractFile {
 	 *
 	 * @return meta data flags
 	 */
-	public long getMeta_flags() {
+	public short getMeta_flags() {
 		return meta_flags;
 	}
 
@@ -373,7 +373,7 @@ public abstract class FsContent extends AbstractFile {
 	 *
 	 * @return user id
 	 */
-	public long getUid() {
+	public int getUid() {
 		return uid;
 	}
 
@@ -382,7 +382,7 @@ public abstract class FsContent extends AbstractFile {
 	 *
 	 * @return group id
 	 */
-	public long getGid() {
+	public int getGid() {
 		return gid;
 	}
 
@@ -391,7 +391,7 @@ public abstract class FsContent extends AbstractFile {
 	 *
 	 * @return mode
 	 */
-	public long getMode() {
+	public short getMode() {
 		return mode;
 	}
 
@@ -423,15 +423,7 @@ public abstract class FsContent extends AbstractFile {
 		return this.parent_path;
 	}
 
-	/**
-	 * Get the absolute unique across all files in the case parent path string
-	 * of this FsContent The path contains image and volume-system partition
-	 * After first call, every subsequent call returns the cached string
-	 *
-	 * @return unique absolute file path (cached after first call)
-	 * @throws TskCoreException thrown when critical error occurred in Tsk Core
-	 * and unique absolute path could not be queried
-	 */
+	@Override
 	public String getUniquePath() throws TskCoreException {
 		if (unique_path != null) {
 			return unique_path;
@@ -525,7 +517,7 @@ public abstract class FsContent extends AbstractFile {
 	 * @param dirType to convert
 	 * @return dir type value string representation
 	 */
-	public static String dirTypeToValue(long dirType) {
+	public static String dirTypeToValue(short dirType) {
 
 		String result = "";
 
@@ -543,9 +535,9 @@ public abstract class FsContent extends AbstractFile {
 	 * @param dirType value string to convert
 	 * @return directory type id
 	 */
-	public static long valueToDirType(String dirType) {
+	public static short valueToDirType(String dirType) {
 
-		long result = 0;
+		short result = 0;
 
 		for (TskData.TSK_FS_NAME_TYPE_ENUM type : TskData.TSK_FS_NAME_TYPE_ENUM.values()) {
 			if (type.toString().equals(dirType)) {
@@ -561,7 +553,7 @@ public abstract class FsContent extends AbstractFile {
 	 * @param dirType dir type id to convert
 	 * @return dir type label string
 	 */
-	public static String dirTypeToString(long dirType) {
+	public static String dirTypeToString(short dirType) {
 		return TskData.TSK_FS_NAME_TYPE_ENUM.fromType(dirType).getLabel();
 	}
 
@@ -572,7 +564,7 @@ public abstract class FsContent extends AbstractFile {
 	 * @param metaType to convert
 	 * @return string value representation of meta type
 	 */
-	public static String metaTypeToValue(long metaType) {
+	public static String metaTypeToValue(short metaType) {
 
 		String result = "";
 
@@ -590,9 +582,9 @@ public abstract class FsContent extends AbstractFile {
 	 * @param metaType to convert
 	 * @return meta type id
 	 */
-	public static long valueToMetaType(String metaType) {
+	public static short valueToMetaType(String metaType) {
 
-		long result = 0;
+		short result = 0;
 
 		for (TskData.TSK_FS_META_TYPE_ENUM type : TskData.TSK_FS_META_TYPE_ENUM.values()) {
 			if (type.toString().equals(metaType)) {
@@ -608,8 +600,8 @@ public abstract class FsContent extends AbstractFile {
 	 * @param metaType to convert
 	 * @return string representation of the meta type
 	 */
-	public static String metaTypeToString(long metaType) {
-		return TskData.tsk_fs_meta_type_str[(int) metaType];
+	public static String metaTypeToString(short metaType) {
+		return TskData.tsk_fs_meta_type_str[metaType];
 	}
 
 // ----- Methods for Directory Flags conversion / mapping -----
@@ -619,7 +611,7 @@ public abstract class FsContent extends AbstractFile {
 	 * @param dirFlag to convert
 	 * @return dir flags string representation
 	 */
-	public static String dirFlagToValue(long dirFlag) {
+	public static String dirFlagToValue(short dirFlag) {
 
 		String result = "";
 
@@ -637,9 +629,9 @@ public abstract class FsContent extends AbstractFile {
 	 * @param dirFlag to convert
 	 * @return dir flag id
 	 */
-	public static long valueToDirFlag(String dirFlag) {
+	public static short valueToDirFlag(String dirFlag) {
 
-		long result = 0;
+		short result = 0;
 
 		for (TskData.TSK_FS_NAME_FLAG_ENUM flag : TskData.TSK_FS_NAME_FLAG_ENUM.values()) {
 			if (flag.toString().equals(dirFlag)) {
@@ -655,12 +647,12 @@ public abstract class FsContent extends AbstractFile {
 	 * @param dirFlag dir flags id to convert
 	 * @return formatted user-readable string representation of dir flag
 	 */
-	public static String dirFlagToString(long dirFlag) {
+	public static String dirFlagToString(short dirFlag) {
 
 		String result = "";
 
-		long allocFlag = TskData.TSK_FS_NAME_FLAG_ENUM.TSK_FS_NAME_FLAG_ALLOC.getDirFlag();
-		long unallocFlag = TskData.TSK_FS_NAME_FLAG_ENUM.TSK_FS_NAME_FLAG_UNALLOC.getDirFlag();
+		short allocFlag = TskData.TSK_FS_NAME_FLAG_ENUM.TSK_FS_NAME_FLAG_ALLOC.getDirFlag();
+		short unallocFlag = TskData.TSK_FS_NAME_FLAG_ENUM.TSK_FS_NAME_FLAG_UNALLOC.getDirFlag();
 
 		if ((dirFlag & allocFlag) == allocFlag) {
 			result = "Allocated";
@@ -679,7 +671,7 @@ public abstract class FsContent extends AbstractFile {
 	 * @param metaFlag to convert
 	 * @return string representation
 	 */
-	public static String metaFlagToValue(long metaFlag) {
+	public static String metaFlagToValue(short metaFlag) {
 
 		String result = "";
 
@@ -692,14 +684,14 @@ public abstract class FsContent extends AbstractFile {
 	}
 
 	/**
-	 * Convert string representation of meta flags to long
+	 * Convert string representation of meta flags to short
 	 *
 	 * @param metaFlag string to convert
-	 * @return long meta flag representation
+	 * @return short meta flag representation
 	 */
-	public static long valueToMetaFlag(String metaFlag) {
+	public static short valueToMetaFlag(String metaFlag) {
 
-		long result = 0;
+		short result = 0;
 
 		for (TskData.TSK_FS_META_FLAG_ENUM flag : TskData.TSK_FS_META_FLAG_ENUM.values()) {
 			if (flag.toString().equals(metaFlag)) {
@@ -715,18 +707,18 @@ public abstract class FsContent extends AbstractFile {
 	 * @param metaFlag to convert
 	 * @return string formatted meta flag representation
 	 */
-	public static String metaFlagToString(long metaFlag) {
+	public static String metaFlagToString(short metaFlag) {
 
 		String result = "";
 
-		long allocFlag = TskData.TSK_FS_META_FLAG_ENUM.ALLOC.getMetaFlag();
-		long unallocFlag = TskData.TSK_FS_META_FLAG_ENUM.UNALLOC.getMetaFlag();
+		short allocFlag = TskData.TSK_FS_META_FLAG_ENUM.ALLOC.getMetaFlag();
+		short unallocFlag = TskData.TSK_FS_META_FLAG_ENUM.UNALLOC.getMetaFlag();
 
 		// some variables that might be needed in the future
-		long usedFlag = TskData.TSK_FS_META_FLAG_ENUM.USED.getMetaFlag();
-		long unusedFlag = TskData.TSK_FS_META_FLAG_ENUM.UNUSED.getMetaFlag();
-		long compFlag = TskData.TSK_FS_META_FLAG_ENUM.COMP.getMetaFlag();
-		long orphanFlag = TskData.TSK_FS_META_FLAG_ENUM.ORPHAN.getMetaFlag();
+		//long usedFlag = TskData.TSK_FS_META_FLAG_ENUM.USED.getMetaFlag();
+		//long unusedFlag = TskData.TSK_FS_META_FLAG_ENUM.UNUSED.getMetaFlag();
+		//long compFlag = TskData.TSK_FS_META_FLAG_ENUM.COMP.getMetaFlag();
+		//long orphanFlag = TskData.TSK_FS_META_FLAG_ENUM.ORPHAN.getMetaFlag();
 
 		if ((metaFlag & allocFlag) == allocFlag) {
 			result = TskData.TSK_FS_META_FLAG_ENUM.ALLOC.getLabel();
@@ -746,30 +738,30 @@ public abstract class FsContent extends AbstractFile {
 	 * @param metaType meta type attribute of the file/dir
 	 * @return converted, formatted user-displayable string
 	 */
-	public static String modeToString(long mode, long metaType) {
+	public static String modeToString(short mode, short metaType) {
 
 		String result = "";
 
-		long metaTypeMax = TskData.TSK_FS_META_TYPE_ENUM.TSK_FS_META_TYPE_STR_MAX.getMetaType();
+		int metaTypeMax = TskData.TSK_FS_META_TYPE_ENUM.TSK_FS_META_TYPE_STR_MAX.getMetaType() & 0xff;
 
-		long isuid = TskData.TSK_FS_META_MODE_ENUM.TSK_FS_META_MODE_ISUID.getMode();
-		long isgid = TskData.TSK_FS_META_MODE_ENUM.TSK_FS_META_MODE_ISGID.getMode();
-		long isvtx = TskData.TSK_FS_META_MODE_ENUM.TSK_FS_META_MODE_ISVTX.getMode();
+		short isuid = TskData.TSK_FS_META_MODE_ENUM.TSK_FS_META_MODE_ISUID.getMode();
+		short isgid = TskData.TSK_FS_META_MODE_ENUM.TSK_FS_META_MODE_ISGID.getMode();
+		short isvtx = TskData.TSK_FS_META_MODE_ENUM.TSK_FS_META_MODE_ISVTX.getMode();
 
-		long irusr = TskData.TSK_FS_META_MODE_ENUM.TSK_FS_META_MODE_IRUSR.getMode();
-		long iwusr = TskData.TSK_FS_META_MODE_ENUM.TSK_FS_META_MODE_IWUSR.getMode();
-		long ixusr = TskData.TSK_FS_META_MODE_ENUM.TSK_FS_META_MODE_IXUSR.getMode();
+		short irusr = TskData.TSK_FS_META_MODE_ENUM.TSK_FS_META_MODE_IRUSR.getMode();
+		short iwusr = TskData.TSK_FS_META_MODE_ENUM.TSK_FS_META_MODE_IWUSR.getMode();
+		short ixusr = TskData.TSK_FS_META_MODE_ENUM.TSK_FS_META_MODE_IXUSR.getMode();
 
-		long irgrp = TskData.TSK_FS_META_MODE_ENUM.TSK_FS_META_MODE_IRGRP.getMode();
-		long iwgrp = TskData.TSK_FS_META_MODE_ENUM.TSK_FS_META_MODE_IWGRP.getMode();
-		long ixgrp = TskData.TSK_FS_META_MODE_ENUM.TSK_FS_META_MODE_IXGRP.getMode();
+		short irgrp = TskData.TSK_FS_META_MODE_ENUM.TSK_FS_META_MODE_IRGRP.getMode();
+		short iwgrp = TskData.TSK_FS_META_MODE_ENUM.TSK_FS_META_MODE_IWGRP.getMode();
+		short ixgrp = TskData.TSK_FS_META_MODE_ENUM.TSK_FS_META_MODE_IXGRP.getMode();
 
-		long iroth = TskData.TSK_FS_META_MODE_ENUM.TSK_FS_META_MODE_IROTH.getMode();
-		long iwoth = TskData.TSK_FS_META_MODE_ENUM.TSK_FS_META_MODE_IWOTH.getMode();
-		long ixoth = TskData.TSK_FS_META_MODE_ENUM.TSK_FS_META_MODE_IXOTH.getMode();
+		short iroth = TskData.TSK_FS_META_MODE_ENUM.TSK_FS_META_MODE_IROTH.getMode();
+		short iwoth = TskData.TSK_FS_META_MODE_ENUM.TSK_FS_META_MODE_IWOTH.getMode();
+		short ixoth = TskData.TSK_FS_META_MODE_ENUM.TSK_FS_META_MODE_IXOTH.getMode();
 
 		// first character = the Meta Type
-		if (metaType < metaTypeMax) {
+		if ( (((int)metaType) & 0xff) < metaTypeMax) {
 			result += FsContent.metaTypeToString(metaType);
 		} else {
 			result += "-";
