@@ -495,19 +495,21 @@ fatfs_block_walk(TSK_FS_INFO * fs, TSK_DADDR_T a_start_blk,
         /* Read 8 sectors at a time to be faster */
         for (; addr < fatfs->firstclustsect && addr <= a_end_blk;) {
 
-            cnt =
-                tsk_fs_read_block(fs, addr, data_buf, fs->block_size * 8);
-            if (cnt != fs->block_size * 8) {
-                if (cnt >= 0) {
-                    tsk_error_reset();
-                    tsk_error_set_errno(TSK_ERR_FS_READ);
+            if ((a_flags & TSK_FS_BLOCK_WALK_FLAG_AONLY) == 0) {
+                cnt =
+                    tsk_fs_read_block(fs, addr, data_buf, fs->block_size * 8);
+                if (cnt != fs->block_size * 8) {
+                    if (cnt >= 0) {
+                        tsk_error_reset();
+                        tsk_error_set_errno(TSK_ERR_FS_READ);
+                    }
+                    tsk_error_set_errstr2
+                        ("fatfs_block_walk: pre-data area block: %" PRIuDADDR,
+                        addr);
+                    free(data_buf);
+                    tsk_fs_block_free(fs_block);
+                    return 1;
                 }
-                tsk_error_set_errstr2
-                    ("fatfs_block_walk: pre-data area block: %" PRIuDADDR,
-                    addr);
-                free(data_buf);
-                tsk_fs_block_free(fs_block);
-                return 1;
             }
 
             /* Process the sectors until we get to the clusters, 
@@ -535,6 +537,8 @@ fatfs_block_walk(TSK_FS_INFO * fs, TSK_DADDR_T a_start_blk,
                     && (!(a_flags & TSK_FS_BLOCK_WALK_FLAG_CONT)))
                     continue;
 
+                if (a_flags & TSK_FS_BLOCK_WALK_FLAG_AONLY)
+                    myflags |= TSK_FS_BLOCK_FLAG_AONLY;
 
                 tsk_fs_block_set(fs, fs_block, addr,
                     myflags | TSK_FS_BLOCK_FLAG_RAW,
@@ -619,25 +623,30 @@ fatfs_block_walk(TSK_FS_INFO * fs, TSK_DADDR_T a_start_blk,
             && (!(a_flags & TSK_FS_BLOCK_WALK_FLAG_UNALLOC)))
             continue;
 
+        if (a_flags & TSK_FS_BLOCK_WALK_FLAG_AONLY)
+            myflags |= TSK_FS_BLOCK_FLAG_AONLY;
 
+        
         /* The final cluster may not be full */
         if (a_end_blk - addr + 1 < fatfs->csize)
             read_size = (size_t) (a_end_blk - addr + 1);
         else
             read_size = fatfs->csize;
 
-        cnt = tsk_fs_read_block
-            (fs, addr, data_buf, fs->block_size * read_size);
-        if (cnt != fs->block_size * read_size) {
-            if (cnt >= 0) {
-                tsk_error_reset();
-                tsk_error_set_errno(TSK_ERR_FS_READ);
+        if ((a_flags & TSK_FS_BLOCK_WALK_FLAG_AONLY) == 0) {
+            cnt = tsk_fs_read_block
+                (fs, addr, data_buf, fs->block_size * read_size);
+            if (cnt != fs->block_size * read_size) {
+                if (cnt >= 0) {
+                    tsk_error_reset();
+                    tsk_error_set_errno(TSK_ERR_FS_READ);
+                }
+                tsk_error_set_errstr2("fatfs_block_walk: block: %" PRIuDADDR,
+                    addr);
+                free(data_buf);
+                tsk_fs_block_free(fs_block);
+                return 1;
             }
-            tsk_error_set_errstr2("fatfs_block_walk: block: %" PRIuDADDR,
-                addr);
-            free(data_buf);
-            tsk_fs_block_free(fs_block);
-            return 1;
         }
 
         /* go through each sector in the cluster */
@@ -1310,16 +1319,14 @@ static void
 fatfs_close(TSK_FS_INFO * fs)
 {
     FATFS_INFO *fatfs = (FATFS_INFO *) fs;
+ 
+    fatfs_dir_buf_free(fatfs);
+
     fs->tag = 0;
-
-    if (fatfs->dir_buf)
-        free(fatfs->dir_buf);
-    if (fatfs->par_buf)
-        free(fatfs->par_buf);
-
     free(fatfs->sb);
     tsk_deinit_lock(&fatfs->cache_lock);
     tsk_deinit_lock(&fatfs->dir_lock);
+	
     tsk_fs_free(fs);
 }
 
@@ -1419,7 +1426,7 @@ fatfs_open(TSK_IMG_INFO * img_info, TSK_OFF_T offset,
                 tsk_error_set_errno(TSK_ERR_FS_MAGIC);
                 tsk_error_set_errstr("Not a FATFS file system (magic)");
                 if (tsk_verbose)
-                    fprintf(stderr, "Not a FATFS file system (magic)");
+                    fprintf(stderr, "fatfs_open: Incorrect FATFS magic\n");
                 return NULL;
             }
         }
@@ -1834,6 +1841,7 @@ fatfs_open(TSK_IMG_INFO * img_info, TSK_OFF_T offset,
     // initialize the caches
     tsk_init_lock(&fatfs->cache_lock);
     tsk_init_lock(&fatfs->dir_lock);
+    fatfs->inum2par = NULL;
 
     return (fs);
 }
