@@ -5,7 +5,7 @@
 ** Given an image  and block number, identify which inode it is used by
 ** 
 ** Brian Carrier [carrier <at> sleuthkit [dot] org]
-** Copyright (c) 2006-2010 Brian Carrier, Basis Technology.  All Rights reserved
+** Copyright (c) 2006-2011 Brian Carrier, Basis Technology.  All Rights reserved
 ** Copyright (c) 2003-2005 Brian Carrier.  All rights reserved
 **
 ** TASK
@@ -241,9 +241,9 @@ tsk_fs_path2inum(TSK_FS_INFO * a_fs, const char *a_path,
          * and this caused problems... */
         if (fs_dir->fs_file->meta->type != TSK_FS_META_TYPE_DIR) {
             tsk_error_reset();
-            tsk_errno = TSK_ERR_FS_GENFS;
-            snprintf(tsk_errstr, TSK_ERRSTR_L, "Address %"PRIuINUM" is not for a directory\n",
-                    next_meta);
+            tsk_error_set_errno(TSK_ERR_FS_GENFS);
+            tsk_error_set_errstr("Address %" PRIuINUM
+                " is not for a directory\n", next_meta);
             free(cpath);
             return -1;
         }
@@ -445,9 +445,9 @@ tsk_fs_ifind_path(TSK_FS_INFO * fs, TSK_TCHAR * tpath, TSK_INUM_T * result)
             (UTF8 *) ((uintptr_t) ptr8 + clen), TSKlenientConversion);
         if (retval != TSKconversionOK) {
             tsk_error_reset();
-            tsk_errno = TSK_ERR_FS_UNICODE;
-            snprintf(tsk_errstr, TSK_ERRSTR_L,
-                "tsk_fs_ifind_path: Error converting path to UTF-8: %d",
+            tsk_error_set_errno(TSK_ERR_FS_UNICODE);
+            tsk_error_set_errstr
+                ("tsk_fs_ifind_path: Error converting path to UTF-8: %d",
                 retval);
             free(cpath);
             return -1;
