@@ -2,7 +2,7 @@
  * The Sleuth Kit
  *
  * Brian Carrier [carrier <at> sleuthkit [dot] org]
- * Copyright (c) 2003-2008 Brian Carrier.  All rights reserved
+ * Copyright (c) 2003-2011 Brian Carrier.  All rights reserved
  *
  * tsk_vs_open - wrapper function for specific partition type
  *
@@ -64,8 +64,8 @@ tsk_vs_open(TSK_IMG_INFO * img_info, TSK_DADDR_T offset,
                vs_set->close(vs_set);
                vs->close(vs);
                tsk_error_reset();
-               tsk_errno = TSK_ERR_VS_UNKTYPE;
-               snprintf(tsk_errstr, TSK_ERRSTR_L,
+               tsk_error_set_errno(TSK_ERR_VS_UNKTYPE);
+               tsk_error_set_errstr(
                "BSD or %s at %" PRIuDADDR, set, offset);
                tsk_errstr2[0] = '\0';
                return NULL;
@@ -85,25 +85,16 @@ tsk_vs_open(TSK_IMG_INFO * img_info, TSK_DADDR_T offset,
                     for (tmp_set = vs_set->part_list; tmp_set;
                         tmp_set = tmp_set->next) {
                         if ((tmp_set->desc)
-                            && (strncmp(tmp_set->desc, "GPT", 3) == 0)
+                            && (strncmp(tmp_set->desc, "GPT Safety",
+                                    10) == 0)
                             && (tmp_set->start <= 63)) {
-                            TSK_VS_PART_INFO *tmp_cur;
-                            /* see if we can find a GPT partition that ends at the same
-                             * location as the DOS (we should be testing for the last GPT partition...) */
-                            for (tmp_cur = vs->part_list; tmp_cur;
-                                tmp_cur = tmp_cur->next) {
-                                if ((tmp_set->start + tmp_set->len) ==
-                                    (tmp_cur->start + tmp_cur->len)) {
-                                    if (tsk_verbose)
-                                        tsk_fprintf(stderr,
-                                            "mm_open: Ignoring DOS Safety GPT Partition\n");
-                                    set = NULL;
-                                    vs_set = NULL;
-                                    break;
-                                }
-                            }
-                            if (set == NULL)
-                                break;
+
+                            if (tsk_verbose)
+                                tsk_fprintf(stderr,
+                                    "mm_open: Ignoring DOS Safety GPT Partition\n");
+                            set = NULL;
+                            vs_set = NULL;
+                            break;
                         }
                     }
                 }
@@ -112,9 +103,9 @@ tsk_vs_open(TSK_IMG_INFO * img_info, TSK_DADDR_T offset,
                     vs_set->close(vs_set);
                     vs->close(vs);
                     tsk_error_reset();
-                    tsk_errno = TSK_ERR_VS_UNKTYPE;
-                    snprintf(tsk_errstr, TSK_ERRSTR_L,
-                        "GPT or %s at %" PRIuDADDR, set, offset);
+                    tsk_error_set_errno(TSK_ERR_VS_UNKTYPE);
+                    tsk_error_set_errstr("GPT or %s at %" PRIuDADDR, set,
+                        offset);
                     return NULL;
                 }
             }
@@ -134,9 +125,9 @@ tsk_vs_open(TSK_IMG_INFO * img_info, TSK_DADDR_T offset,
                 vs_set->close(vs_set);
                 vs->close(vs);
                 tsk_error_reset();
-                tsk_errno = TSK_ERR_VS_UNKTYPE;
-                snprintf(tsk_errstr, TSK_ERRSTR_L,
-                    "Sun or %s at %" PRIuDADDR, set, offset);
+                tsk_error_set_errno(TSK_ERR_VS_UNKTYPE);
+                tsk_error_set_errstr("Sun or %s at %" PRIuDADDR, set,
+                    offset);
                 return NULL;
             }
         }
@@ -153,9 +144,9 @@ tsk_vs_open(TSK_IMG_INFO * img_info, TSK_DADDR_T offset,
                 vs_set->close(vs_set);
                 vs->close(vs);
                 tsk_error_reset();
-                tsk_errno = TSK_ERR_VS_UNKTYPE;
-                snprintf(tsk_errstr, TSK_ERRSTR_L,
-                    "Mac or %s at %" PRIuDADDR, set, offset);
+                tsk_error_set_errno(TSK_ERR_VS_UNKTYPE);
+                tsk_error_set_errstr("Mac or %s at %" PRIuDADDR, set,
+                    offset);
                 return NULL;
             }
         }
@@ -165,7 +156,7 @@ tsk_vs_open(TSK_IMG_INFO * img_info, TSK_DADDR_T offset,
 
         if (vs_set == NULL) {
             tsk_error_reset();
-            tsk_errno = TSK_ERR_VS_UNKTYPE;
+            tsk_error_set_errno(TSK_ERR_VS_UNKTYPE);
             return NULL;
         }
 
@@ -187,8 +178,8 @@ tsk_vs_open(TSK_IMG_INFO * img_info, TSK_DADDR_T offset,
         case TSK_VS_TYPE_UNSUPP:
         default:
             tsk_error_reset();
-            tsk_errno = TSK_ERR_VS_UNSUPTYPE;
-            snprintf(tsk_errstr, TSK_ERRSTR_L, "%d", type);
+            tsk_error_set_errno(TSK_ERR_VS_UNSUPTYPE);
+            tsk_error_set_errstr("%d", type);
             return NULL;
         }
     }
